package ovh.herisson.Clyde.EndPoints;

import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;
import ovh.herisson.Clyde.Repositories.TokenRepository;
import ovh.herisson.Clyde.Repositories.UserRepository;
import ovh.herisson.Clyde.Services.TokenService;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.Token;
import ovh.herisson.Clyde.Tables.User;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;

@RestController
@CrossOrigin(origins = "http://localhost:5173")

public class MockController {
    private final BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public final UserRepository userRepo;
    public final TokenRepository tokenRepo;
    public final TokenService tokenService;
    ArrayList<User> mockUsers;


    public MockController(UserRepository userRepo, TokenRepository tokenRepo, TokenService tokenService){
        this.tokenRepo = tokenRepo;
        this.userRepo = userRepo;
        this.tokenService = tokenService;
    }

    /** Saves an example of each user type by :
     * email : FooRole@FooRole.com, password : FooRole and token : FooRole
     * For example the admin as "admin@admin.com" as email and "admin" as both password and token
     * They all have silly names
     */

    @PostMapping("/mock")
    public void postMock(){

        User herobrine = new User("brine","hero","admin@admin.com","in your WalLs","ShadowsLand",new Date(0), null,Role.Admin,passwordEncoder.encode("admin"));
        User joe = new User("Mama","Joe","student@student.com","roundabout","DaWarudo",new Date(0), null,Role.Student,passwordEncoder.encode("student"));
        User meh = new User("Inspiration","lackOf","secretary@secretary.com","a Box","the street",new Date(0), null,Role.Teacher,passwordEncoder.encode("secretary"));
        User joke = new User("CthemBalls","Lemme","teacher@teacher.com","lab","faculty",new Date(0), null,Role.Teacher,passwordEncoder.encode("teacher"));

        mockUsers = new ArrayList<User>(Arrays.asList(herobrine,joe,meh,joke));

        userRepo.saveAll(mockUsers);

        for (User user: mockUsers){
            //Petit test pour être sur de delete les bons tokens
            if (user.getLastName().equals("brine")){
                Calendar c = Calendar.getInstance();
                c.setTime(new Date());
                c.add(Calendar.DAY_OF_WEEK, 1);
                tokenService.saveToken(new Token(user,user.getPassword(), c.getTime()));
            }else{
                tokenService.saveToken(new Token(user,user.getPassword(), new Date()));
            }

        }
    }

    //Nous allons donner des tokens a herobrine pour tester la limite de token
    @PostMapping("/herobrine")
    public void giveHerobrineTokens(){
        User herobrine = userRepo.findById(1);
        Calendar c = Calendar.getInstance();
        for (int i = 1; i <= 7; i++){
            Token t = new Token(herobrine, herobrine.getPassword(), c.getTime());
            c.add(Calendar.DAY_OF_YEAR, 1);
            tokenService.saveToken(t);
        }
    }
    @DeleteMapping("/mock")
    public void deleteMock(){
        for (User user:mockUsers){
            tokenRepo.deleteAll(tokenRepo.getByUser(user));
        }
        userRepo.deleteAll(mockUsers);
    }
}
